
#include <stdio.h>
#include <string.h>

#include <nds.h>

#include <memory.h>
#include <ARM9/console.h>

static void myCardReadEeprom(uint8 *data, uint32 length) ;
static void myCardWriteEeprom(const uint8 *apData, uint32 aLength);

#define BUFFER (MAINRAM8 + 0x00100000)

int main(void)
{
    bool keyPressed = FALSE;

	videoSetMode(0);
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);
	vramSetBankC(VRAM_C_SUB_BG);
	SUB_BG0_CR = BG_MAP_BASE(31);
	BG_PALETTE_SUB[255] = RGB15(31,31,31);
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);

    while (1)
    {
	    consolePrintSet(0,2);
	    consolePrintf("A - Backup EEPROM to SRAM.\n"
	                  "B - Restore EEPROM from SRAM.\n"
	                  "SELECT - Format the EEPROM (FF).");

	    consolePrintSet(0,21);
	    consolePrintf("Use this program at your own\n"
	                  "risk. Created by Brian Bennett");

        /* No key currently pressed */
        if (!keyPressed)
        {
            /* A Pressed */
		    if (!(KEYS & KEY_A)) 
            {
                int i;
                int length = 64 * 1024;
    
        	    consolePrintSet(0,5);
        		consolePrintf("                                ");
        		consolePrintf("                                ");
        		consolePrintf("                                ");
        		consolePrintf("                                ");
        		consolePrintf("                                ");

                /* Enable the ARM9 to access the GBA cartridge memory area and the NDS cartridge. */
                WAIT_CR &= ~0x0880;

            	myCardReadEeprom(BUFFER, length);

                for (i = 0; i < length; i++)
                {
                    BUFFER[i] = SRAM[i];
                }

        	    consolePrintSet(0,5);
                consolePrintf("Backup Finished");
                keyPressed = TRUE;

            }
            /* B Pressed */
		    else if (!(KEYS & KEY_B)) 
            {
                int i;
                int length = 64 * 1024;
                
        	    consolePrintSet(0,5);
        		consolePrintf("                                ");
        		consolePrintf("                                ");
        		consolePrintf("                                ");
        		consolePrintf("                                ");
        		consolePrintf("                                ");

                /* Enable the ARM9 to access the GBA cartridge memory area and the NDS cartridge. */
                WAIT_CR &= ~0x0880;

        	    consolePrintSet(0,6);

                for (i = 0; i < 20; i++)
                {
                    consolePrintf("%02x ", SRAM[i]);
                }

                for (i = 0; i < length; i++)
                {
                    BUFFER[i] = SRAM[i];
                }

            	myCardWriteEeprom(BUFFER, length);

        	    consolePrintSet(0,5);
                consolePrintf("Write Done      ");
                keyPressed = TRUE;
	        }
            /* Select Pressed */
		    else if (!(KEYS & KEY_SELECT)) 
            {
        	    consolePrintSet(0,5);
        		consolePrintf("                                ");
        		consolePrintf("                                ");
        		consolePrintf("                                ");
        		consolePrintf("                                ");
        		consolePrintf("                                ");

                /* Enable the ARM9 to access the GBA cartridge memory area and the NDS cartridge. */
                WAIT_CR &= ~0x0880;
    
                memset(BUFFER, 0xff, 64 * 1024);
    
            	myCardWriteEeprom(BUFFER, 64 * 1024);
            
        	    consolePrintSet(0,5);
                consolePrintf("Erase Done      ");
                keyPressed = TRUE;
	        }
        }
        /* KEY A and B and select not pressed */
        else if (!(!(KEYS & KEY_A) || !(KEYS & KEY_B) || !(KEYS & KEY_SELECT))) 
        {
            keyPressed = FALSE;
        }
    }
}

static void EepromWaitBusy(void)
{
    while (CARD_CR1 & /*BUSY*/0x80)
    {
        consolePrintSet(0,0);
        consolePrintf("Busy");
    }
    consolePrintSet(0, 0);
    consolePrintf("    ");
}

// NOTE: only tried on 64K eeprom
static void myCardReadEeprom(uint8 *apData, uint32 aLength) 
{
	CARD_CR1 = /*E*/0x8000 | /*SEL*/0x2000 | /*MODE*/0x40;
	CARD_EEPDATA = 0x03; 
    EepromWaitBusy();

    CARD_EEPDATA = 0; 
    EepromWaitBusy(); 

    CARD_EEPDATA = 0; 
    EepromWaitBusy(); 

	while (aLength-- > 0)
	{
		CARD_EEPDATA = 0; 
        EepromWaitBusy();
		*apData = CARD_EEPDATA;
        consolePrintSet(5, 0);
        consolePrintf("%02x ", *apData);
        apData++;
	}
	CARD_CR1 = /*MODE*/0x40;
}

// NOTE: only tried on 64K eeprom
static void myCardWriteEeprom(const uint8 *apData, uint32 aLength)
{
	int i;
    uint32 address = 0;

	while (address < aLength)
	{
		// set WEL (Write Enable Latch)
		CARD_CR1 = /*E*/0x8000 | /*SEL*/0x2000 | /*MODE*/0x40;
		CARD_EEPDATA = 0x06; 
        EepromWaitBusy();
		CARD_CR1 = /*MODE*/0x40;

		// program maximum of 32 bytes
		CARD_CR1 = /*E*/0x8000 | /*SEL*/0x2000 | /*MODE*/0x40;
		CARD_EEPDATA = 0x02; 
        EepromWaitBusy();
        CARD_EEPDATA = address >> 8; 
        EepromWaitBusy(); 
		CARD_EEPDATA = address & 0xFF; 
        EepromWaitBusy();
		for (i = 0; address < aLength && i < 32; i++, address++) 
        { 
            CARD_EEPDATA = *apData; 
            EepromWaitBusy(); 
            consolePrintSet(5, 0);
            consolePrintf("%02x ", *apData);
            apData++;
        }
		CARD_CR1 = /*MODE*/0x40;

		// wait programming to finish
		CARD_CR1 = /*E*/0x8000 | /*SEL*/0x2000 | /*MODE*/0x40;
		CARD_EEPDATA = 0x05; 
        EepromWaitBusy();
		do 
        { 
            CARD_EEPDATA = 0; 
            EepromWaitBusy(); 
        } while (CARD_EEPDATA & 0x01);	// WIP (Write In Progress) ?
		CARD_CR1 = /*MODE*/0x40;
	}
}
